import { NgModule, ModuleWithProviders } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { KendoUiAngularModule } from './shared/modules/kendo-ui-angular/kendo-ui-angular.module';
import { TranslateModule, TranslateLoader } from '@ngx-translate/core';
import { TranslateHttpLoader } from '@ngx-translate/http-loader';

// Services
import { SiteService } from './shared/services/site.service';
import { AtsI18nService } from './shared/services/ats.i18n.service';
import { PermissionsService } from './shared/services/permissions.service';

// Components
import { CoreFoundationComponent } from './core-foundation.component';
import { AppBarComponent } from './shared/app/app-bar/app-bar.component';
import { SideBarComponent } from './shared/app/side-bar/side-bar.component';
import { LoadOverlayComponent } from './shared/components/load-overlay/load-overlay.component';
import { AppoverlayComponent } from './shared/components/appoverlay/appoverlay.component';
import { DateFormatPipe } from './shared/pipes/core-date-format.pipe';
import { FilterdataPipe } from './shared/pipes/search-by-name.pipe';
import { ScriptsService } from './shared/scripts/scripts.service';

// AoT requires an exported function for factories
export function HttpLoaderFactory(httpClient: HttpClient) {
    return new TranslateHttpLoader(httpClient);
}

@NgModule({
    declarations: [
        CoreFoundationComponent,
        LoadOverlayComponent,
        AppBarComponent,
        SideBarComponent,
        AppoverlayComponent,
        DateFormatPipe,
        FilterdataPipe
    ],
    imports: [
        KendoUiAngularModule,
        TranslateModule.forRoot({
            loader: {
                provide: TranslateLoader,
                //useFactory: HttpLoaderFactory,
                useFactory: HttpLoaderFactory,     // (http: any) => new TranslateHttpLoader(http),

                deps: [HttpClient]
            }
        })
    ],
    providers: [
        // SiteService, // To use core library service in client app and other library projects we need to provide services in here instead of { providedIn: 'root' }
        AtsI18nService,
        PermissionsService
       ,ScriptsService
    ],
    exports: [
        KendoUiAngularModule,
        CoreFoundationComponent,
        LoadOverlayComponent,
        AppBarComponent,
        SideBarComponent,
        AppoverlayComponent,
        TranslateModule,
        DateFormatPipe,
        FilterdataPipe

    ]
})
export class CoreFoundationModule {
    // use static services as needed here
    static forRoot(): ModuleWithProviders {
        return {
            ngModule: CoreFoundationModule,
            providers: [SiteService]
        };
    }

}
