﻿import { Injectable } from "@angular/core";
import { ScriptStore } from "./script.store";
import { BehaviorSubject, Observable, Subject } from 'rxjs';

declare var document: any;

@Injectable()
export class ScriptsService {
    private angularData = new Subject<any>();
    private scripts: any = {};
    private allScripts: string[] = [];

    // private arrScripts: string[] = [];

    constructor() {
        ScriptStore.forEach((script: any) => {
            // this.arrScripts.push(script.src);
            this.allScripts.push(script.name);

            this.scripts[script.name] = {
                loaded: false,
                src: script.src
            };
        });
    }

    load(...scripts: string[]) {
        var promises: any[] = [];
        scripts.forEach((script) => promises.push(this.loadScript(script)));
        return Promise.all(promises);
    }

    loadFoundationScripts() {
        var promises: any[] = [];
        this.allScripts.forEach((script) => promises.push(this.loadScript(script)));
        return Promise.all(promises);
    }

    loadFoundationScriptsToDelete() {
        //for (let i = 0; i < this.arrScripts.length; i++) {
        //    const node = document.createElement('script');
        //    node.src = this.arrScripts[i];
        //    node.type = 'text/javascript';
        //    node.async = false;
        //    node.defer = 'defer';
        //    node.charset = 'utf-8';
        //    document.getElementsByTagName('head')[0].appendChild(node);            
        //}

        ScriptStore.forEach((script: any) => {
            let scrpt = document.createElement('script');
            scrpt.type = 'text/javascript';
            scrpt.src = script.src;
            scrpt.defer = "defer";
            document.getElementsByTagName('head')[0].appendChild(scrpt);
        });
    }

    loadScript(name: string) {
        return new Promise((resolve, reject) => {
          
            //resolve if already loaded
            if (this.scripts[name].loaded) {
                resolve({ script: name, loaded: true, status: 'Already Loaded' });
            }
            else {
                //load script
                let script = document.createElement('script');
                script.type = 'text/javascript';
                script.src = this.scripts[name].src;
                script.defer = "defer";
                if (script.readyState) {  //IE
                    script.onreadystatechange = () => {
                        if (script.readyState === "loaded" || script.readyState === "complete") {
                            script.onreadystatechange = null;
                            this.scripts[name].loaded = true;
                            resolve({ script: name, loaded: true, status: 'Loaded' });
                        }
                    };
                } else {  //Others
                    script.onload = () => {
                        this.scripts[name].loaded = true;
                        resolve({ script: name, loaded: true, status: 'Loaded' });
                    };
                }
                script.onerror = (error: any) => resolve({ script: name, loaded: false, status: 'Loaded' });
                document.getElementsByTagName('head')[0].appendChild(script);
            }
        });
    }

    setAngularData(data: any) {
        this.angularData.next(data);
    }

    getAngularData(): Observable<any> {
        return this.angularData.asObservable();
    }

}